@@ -178,30 +178,6 @@ web site (http://www.koha.org/).
 
 |;
 
-$messages->{'NETZ3950Missing'}->{en}=qq|
-
-The Net::Z3950 module is missing.  This module is necessary if you want to use
-Koha's Z39.50 client to download bibliographic records from other libraries.
-
-To install this module, you will need the yaz client installed from
-http://www.indexdata.dk/yaz/ and then you can install the perl module with the
-command:
-
-perl -MCPAN -e 'install Net::Z3950'
-
-...or by installing packages for your distribution, if available.
-
-IMPORTANT NOTE : If you use Perl 5.8.0, you might need to 
-edit NET::Z3950's Makefile.PL and yazwrap/Makefile.PL to include:
-
-    'DEFINE' => '-D_GNU_SOURCE',
-
-Also note that some installations of Perl on Red Hat will generate a lot of
-"'my_perl' undeclared" errors when running make in Net-Z3950.  This is fixed by
-inserting in yazwrap/ywpriv.h a line saying #include "XSUB.h"
-
-Press the <ENTER> key to continue: |;	#'
-
 $messages->{'CheckingPerlModules'}->{en} = heading('PERL MODULES') . qq|
 Checking perl modules ...
 |;
@@ -859,29 +835,44 @@ sub checkperlmodules {
 	unless (eval {require Digest::MD5})      { push @missing,"Digest::MD5" };
 	unless (eval {require MARC::Record})     { push @missing,"MARC::Record" };
 	unless (eval {require Mail::Sendmail})   { push @missing,"Mail::Sendmail" };
+	unless (eval {require MARC::Charset})   { push @missing,"MARC::Charset" };
+	unless (eval {require MARC::File::XML})   { push @missing,"MARC::File::XML" };
+	unless (eval {require MARC::File::XML})   { push @missing,"LWP::Simple" };
+	unless (eval {require MARC::File::XML})   { push @missing,"XML::Simple" };
+	unless (eval {require MARC::File::XML})   { push @missing,"Date::Calc" };
+	unless (eval {require MARC::File::XML})   { push @missing,"MARC::File::XML" };
+	unless (eval {require ZOOM})   { push @missing,"ZOOM" };
 # The following modules are not mandatory, depends on how the library want to use Koha
 	unless (eval {require PDF::API2})   { 
 			if ($#missing>=0) { # only when $#missing >= 0 so this isn't fatal
-				push @missing,"You will need PDF::API2 for barcode generator" 
+				push @missing,"You will need PDF::API2 for barcode generator";
+			}
+	}
+	unless (eval {require GD::Barcorde})   { 
+			if ($#missing>=0) { # only when $#missing >= 0 so this isn't fatal
+				push @missing,"You will need GD::Barcode for the new barcode generator";
+			}
+	}
+	unless (eval {require Data::Random})   { 
+			if ($#missing>=0) { # only when $#missing >= 0 so this isn't fatal
+				push @missing,"You will need Data::Random for the new barcode generator";
+			}
+	}
+	unless (eval {require PDF::Reuse::Barcode})   { 
+			if ($#missing>=0) { # only when $#missing >= 0 so this isn't fatal
+				push @missing,"You will need PDF::Reuse::Barcode for the new barcode generator";
 			}
 	}
 	unless (eval {require Net::LDAP})       {
 		if ($#missing>=0) { # only when $#missing >= 0 so this isn't fatal
-				push @missing, "Net::LDAP";
+				push @missing, "You will need Net::LDAP if you want to authentify patrons through a LDAP";
 			}
-    }
-	unless (eval {require Event})       {
-		if ($#missing>=0) { # only when $#missing >= 0 so this isn't fatal
-				push @missing, "Event";
-			}
-    }
-    unless (eval {require Net::Z3950})       {
-		showmessage(getmessage('NETZ3950Missing'), 'PressEnter', '', 1);
-		if ($#missing>=0) { # see above note
-			push @missing, "Net::Z3950";
-		}
-    }
-
+        }
+        unless (eval {require Text::Wrap})       {
+                if ($#missing>=0) { # only when $#missing >= 0 so this isn't fatal
+                                push @missing, "You will need Text::Wrap if you want to use Spine-label printing";
+                        }
+        }
 #
 # Print out a list of any missing modules
 #
@@ -1745,7 +1736,7 @@ $messages->{'MysqlRootPassword'}->{en} =
 To create the koha database, please enter your
 mysql server's root user password:
 
-Password: |;	#'
+Password: |;
 
 $messages->{'CreatingDatabase'}->{en} = heading('CREATING DATABASE') . qq|
 Creating the MySQL database for Koha...
@@ -1809,18 +1800,16 @@ sub databasesetup {
 	setmysqlclipass($mysqlpass);
 	# Set up permissions
 	startsysout();
-	print system("$mysqldir/bin/mysql -u$mysqluser -e \"insert into user (Host,User,Password) values ('$hostname','$user',password('$pass'))\" mysql\;");
-	system("$mysqldir/bin/mysql -u$mysqluser -e \"insert into db (Host,Db,User,Select_priv,Insert_priv,Update_priv,Delete_priv,Create_priv,Drop_priv, index_priv, alter_priv) values ('%','$database','$user','Y','Y','Y','Y','Y','Y','Y','Y')\" mysql");
-	system("$mysqldir/bin/mysqladmin -u$mysqluser reload");
+	my $result=system("$mysqldir/bin/mysqladmin", "-u$mysqluser", "create", "$database");
+	system("$mysqldir/bin/mysql '-u$mysqluser' -e \"GRANT ALL PRIVILEGES on ".$database.".* to '$user' IDENTIFIED BY '$pass' \" mysql");
 	# Change to admin user login
 	setmysqlclipass($pass);
-	my $result=system("$mysqldir/bin/mysqladmin", "-u$user", "create", "$database");
 	if ($result) {
 		showmessage(getmessage('CreatingDatabaseError'),'PressEnter', '', 1);
 	} else {
 		# Create the database structure
 		startsysout();
-		system("$mysqldir/bin/mysql -u$user $database < koha.mysql");
+		system("$mysqldir/bin/mysql '-u$user' $database < koha.mysql");
 	}
 
 }
@@ -1848,8 +1837,10 @@ $messages->{'UpdateMarcTables'}->{en} =
    heading('MARC FIELD DEFINITIONS') . qq|
 You can import MARC settings for:
 
-  1 MARC21
-  2 UNIMARC
+  1 MARC21 in english
+  2 UNIMARC in french
+  3 UNIMARC in english
+  4 UNIMARC in ukrainian
   N none
 
 NOTE: If you choose N,
@@ -1895,14 +1886,20 @@ sub updatedatabase {
 		$response=$auto_install->{UpdateMarcTables};
 		print ON_YELLOW.BLACK."auto-setting UpdateMarcTable to : $response".RESET."\n";
 	} else {
-		$response=showmessage(getmessage('UpdateMarcTables'), 'restrictchar 12Nn', '1');
+		$response=showmessage(getmessage('UpdateMarcTables'), 'restrictchar 1234Nn', '1');
 	}
 	startsysout();
 	if ($response eq '1') {
-		system("cat scripts/misc/marc_datas/marc21_en/structure_def.sql | $mysqldir/bin/mysql -u$user $database");
+		system("cat scripts/misc/marc_datas/marc21_en/structure_def.sql | $mysqldir/bin/mysql '-u$user' $database");
 	}
 	if ($response eq '2') {
-		system("cat scripts/misc/marc_datas/unimarc_fr/structure_def.sql | $mysqldir/bin/mysql -u$user $database");
+		system("cat scripts/misc/marc_datas/unimarc_fr/structure_def.sql | $mysqldir/bin/mysql '-u$user' $database");
+	}
+	if ($response eq '3') {
+		system("cat scripts/misc/marc_datas/unimarc_en/structure_def.sql | $mysqldir/bin/mysql '-u$user' $database");
+	}
+	if ($response eq '4') {
+		system("cat scripts/misc/marc_datas/unimarc_uk/structure_def.sql | $mysqldir/bin/mysql '-u$user' $database");
 	}
 	delete($ENV{"KOHA_CONF"});
 
@@ -1928,17 +1925,20 @@ sub populatedatabase {
 	my $input;
 	my $response;
 	my $branch='MAIN';
+	my $setbranch=0; #MJR: need new test flag, because branch is preset
 	if ($auto_install->{BranchName}) {
 		$branch=$auto_install->{BranchName};
 		print ON_YELLOW.BLACK."auto-setting a branch : $branch".RESET."\n";
+		$setbranch=1;
 	} else {
 		$response=showmessage(getmessage('AddBranchPrinter'), 'yn', 'y');
 		unless ($response =~/^n/i) {
 			$branch=showmessage(getmessage('BranchName', [$branch]), 'free', $branch, 1);
 			$branch=~s/[^A-Za-z0-9\s]//g;
+			$setbranch=1;
 		}
 	}
-	if ($branch) {
+	if ($setbranch==1) {
 		my $branchcode=$branch;
 		$branchcode=~s/[^A-Za-z0-9]//g;
 		$branchcode=uc($branchcode);
@@ -1955,9 +1955,9 @@ sub populatedatabase {
 		$branchcode or $branchcode='DEF';
 
 		startsysout();
-		system("$mysqldir/bin/mysql -u$user -e \"insert into branches (branchcode,branchname,issuing) values ('$branchcode', '$branch', 1)\" $database");
-		system("$mysqldir/bin/mysql -u$user -e \"insert into branchrelations (branchcode,categorycode) values ('MAIN', 'IS')\" $database");
-		system("$mysqldir/bin/mysql -u$user -e \"insert into branchrelations (branchcode,categorycode) values ('MAIN', 'CU')\" $database");
+		system("$mysqldir/bin/mysql '-u$user' -e \"insert into branches (branchcode,branchname,issuing) values ('$branchcode', '$branch', 1)\" $database");
+		system("$mysqldir/bin/mysql '-u$user' -e \"insert into branchrelations (branchcode,categorycode) values ('MAIN', 'IS')\" $database");
+		system("$mysqldir/bin/mysql '-u$user' -e \"insert into branchrelations (branchcode,categorycode) values ('MAIN', 'CU')\" $database");
 
 		my $printername='lp';
 		my $printerqueue='/dev/lp0';
@@ -1976,7 +1976,7 @@ sub populatedatabase {
 			$printerqueue=~s/[^A-Za-z0-9]//g;
 		}
 		startsysout();	
-		system("$mysqldir/bin/mysql -u$user -e \"insert into printers (printername,printqueue,printtype) values ('$printername', '$printerqueue', '')\" $database");
+		system("$mysqldir/bin/mysql '-u$user' -e \"insert into printers (printername,printqueue,printtype) values ('$printername', '$printerqueue', '')\" $database");
 	}
 	my $language;
 	if ($auto_install->{Language}) {
@@ -1986,7 +1986,7 @@ sub populatedatabase {
 		$language=showmessage(getmessage('Language'), 'free', 'en');
 	}
 	startsysout();	
-	system("$mysqldir/bin/mysql -u$user -e \"update systempreferences set value='$language' where variable='opaclanguages'\" $database");
+	system("$mysqldir/bin/mysql '-u$user' -e \"update systempreferences set value='$language' where variable='opaclanguages'\" $database");
 	my @dirs;
 	if (-d "scripts/misc/sql-datas") {
 		# ask for directory to look for files to append
@@ -2050,7 +2050,7 @@ if you confirm, the file will be added to the DB
 						# if confirmed, upload the file in the DB
 						unless ($response =~/^n/i) {
 							$FileToUpload =~ s/\.txt/\.sql/;
-							system("$mysqldir/bin/mysql -u$user $database <scripts/misc/sql-datas/$sqluploaddir/$FileToUpload");
+							system("$mysqldir/bin/mysql '-u$user' $database <scripts/misc/sql-datas/$sqluploaddir/$FileToUpload");
 						}
 					}
 				}
