@@ -32,8 +32,12 @@ while (my ($description,$itemtype) = $sth->fetchrow) {
 }
 
 $template->param(classlist => $classlist,
-						type => 'intranet',
-		 branches=>\@branches,
-		 itemtypes=>\@itemtypes);
+		type => 'intranet',
+		branches=>\@branches,
+		itemtypes=>\@itemtypes,
+		intranetcolorstylesheet => C4::Context->preference("intranetcolorstylesheet"),
+		intranetstylesheet => C4::Context->preference("intranetstylesheet"),
+		IntranetNav => C4::Context->preference("IntranetNav"),
+		);
 
 output_html_with_http_headers $query, $cookie, $template->output;
