@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-# $Id: issues_avg_stats.pl,v 1.2.2.1 2005/04/07 12:06:05 tipaul Exp $
+# $Id: issues_avg_stats.pl,v 1.2.2.5 2006/08/03 13:33:17 hdl Exp $
 
 # Copyright 2000-2002 Katipo Communications
 #
@@ -64,7 +64,11 @@ my ($template, $borrowernumber, $cookie)
 				flagsrequired => {editcatalogue => 1},
 				debug => 1,
 				});
-$template->param(do_it => $do_it);
+$template->param(do_it => $do_it,
+		intranetcolorstylesheet => C4::Context->preference("intranetcolorstylesheet"),
+		intranetstylesheet => C4::Context->preference("intranetstylesheet"),
+		IntranetNav => C4::Context->preference("IntranetNav"),
+		);
 if ($do_it) {
 # Displaying results
 	my $results = calculate($line, $column, $rodsp, $podsp, $calc, \@filters);
@@ -520,16 +524,19 @@ sub calculate {
 		$emptycol=1 if ($col eq undef);
 		$col = "zzEMPTY" if ($col eq undef);
 		$row = "zzEMPTY" if ($row eq undef);
-#		warn "506 row :".$row." column :".$col;
+# 		warn "506 row :".$row." column :".$col;
 		my @result =split /:/,DateCalc($returndate,$issuedate) ;
 #  DateCalc returns => 0:0:WK:DD:HH:MM:SS   the weeks, days, hours, minutes,
 #  and seconds between the two
+#         warn " dateretour $returndate , date pret $issuedate";
+#         warn " retour de DateCalc :".DateCalc($returndate,$issuedate);
 		$loanlength = $result[2]*7+$result[3];
-#		warn "512 Same row and col DateCalc returns :$loanlength with return ". $returndate ."issue ". $issuedate ."weight : ". $weight;
-#		warn "513 row :".$row." column :".$col;
+# 		warn "512 Same row and col DateCalc returns :$loanlength with return ". $returndate ."issue ". $issuedate ."weight : ". $weight;
+# 		warn "513 row :".$row." column :".$col;
 		$table{$row}->{$col}+=$weight*$loanlength;
-#		$table{$row}->{totalrow}+=$weight*$loanlength;
-		$cnttable{$row}->{$col}= 1;
+# 		warn "".$table{$row}->{$col};
+		$table{$row}->{totalrow}+=$weight*$loanlength;
+# 		$cnttable{$row}->{$col}= 1;
 		$wgttable{$row}->{$col}+=$weight;
 	}
 	
@@ -541,12 +548,16 @@ sub calculate {
 	# and the number matches the number of columns
 		my $colcount=0;
 		foreach my $col ( @loopcol ) {
-			my $value =$table{$row}->{(($col->{coltitle} eq "NULL")or ($col->{coltitle} eq ""))?"zzEMPTY":$col->{coltitle}} / $wgttable{$row}->{(($col->{coltitle} eq "NULL")or ($col->{coltitle} eq ""))?"zzEMPTY":$col->{coltitle}} if ($table{$row}->{(($col->{coltitle} eq "NULL")or ($col->{coltitle} eq ""))?"zzEMPTY":$col->{coltitle}});
+            my $tmpcol = ((($col->{coltitle} eq "NULL")or ($col->{coltitle} eq ""))?"zzEMPTY":$col->{coltitle});
+#             warn "row : $row col:$tmpcol  TABLE : ".$table{$row}->{$tmpcol}." POIDS : ".$wgttable{$row}->{$tmpcol};
+            my $value = $table{$row}->{$tmpcol} / $wgttable{$row}->{$tmpcol} if ($table{$row}->{$tmpcol});
 
-			$table{$row}->{(($col->{coltitle} eq "NULL")or ($col->{coltitle} eq ""))?"zzEMPTY":$col->{coltitle}} = $value;
+			$table{$row}->{$tmpcol} = $value;
 			$table{$row}->{totalrow}+=$value;
-			#warn "row : $row col:$col  $cnttable{$row}->{(($col->{coltitle} eq \"NULL\")or ($col->{coltitle} eq \"\"))?\"zzEMPTY\":$col->{coltitle}}";
-			$colcount+=$cnttable{$row}->{(($col->{coltitle} eq "NULL")or ($col->{coltitle} eq ""))?"zzEMPTY":$col->{coltitle}};
+			
+#             warn "row : $row col:$tmpcol  TABLE : ".$table{$row}->{$tmpcol}." POIDS : ".$wgttable{$row}->{$tmpcol};
+            
+			$colcount+=$cnttable{$row}->{$tmpcol};
 			push @loopcell, {value => ($value)?sprintf("%.2f",$value):0  } ;
 		}
 		#warn "row : $row colcount:$colcount";
@@ -589,4 +600,4 @@ sub calculate {
 	return \@mainloop;
 }
 
-1;
\ No newline at end of file
+1;
