@@ -49,11 +49,12 @@ my $count;
 my $toggle;
 
 my $record;
-my $oldbiblio;
+my $biblionumber;
 if ($bibid > 0) {
 	$record = MARCgetbiblio($dbh,$bibid);
-	$oldbiblio = MARCmarc2koha($dbh,$record);
+	$biblionumber=MARCfind_oldbiblionumber_from_MARCbibid($dbh,$bibid);
 }
+
 my $errmsg;
 unless ($random) { # if random is a parameter => we're just waiting for the search to end, it's a refresh.
 	if ($isbn) {
@@ -76,16 +77,18 @@ my ($template, $loggedinuser, $cookie)
 				debug => 1,
 				});
 
+#MJR: check for pending then search results, else servers with fast 
+# z3950 links may miss results if the search completes between them
+my $numberpending= &checkz3950searchdone($random);
 # fill with books in breeding farm
 ($count, @results) = BreedingSearch($title,$isbn,$random);
-my $numberpending= &checkz3950searchdone($random);
 my @breeding_loop = ();
 for (my $i=0; $i <= $#results; $i++) {
 	my %row_data;
 	if ($i % 2) {
-		$toggle="#ffffcc";
+		$toggle=1;
 	} else {
-		$toggle="white";
+		$toggle=0;
 	}
 	$row_data{toggle} = $toggle;
 	$row_data{id} = $results[$i]->{'id'};
@@ -97,14 +100,16 @@ for (my $i=0; $i <= $#results; $i++) {
 }
 
 $template->param(isbn => $isbn,
-						title => $title,
-						author => $author,
-						breeding_loop => \@breeding_loop,
-						refresh => ($numberpending eq 0 ? 0 : "search.pl?bibid=$bibid&random=$random"),
-						numberpending => $numberpending,
-						oldbiblionumber => $oldbiblio->{'biblionumber'},
-						);
-
+		title => $title,
+		author => $author,
+		breeding_loop => \@breeding_loop,
+		refresh => ($numberpending eq 0 ? "" : "search.pl?bibid=$bibid&random=$random"),
+		numberpending => $numberpending,
+		oldbiblionumber => $biblionumber,
+		intranetcolorstylesheet => C4::Context->preference("intranetcolorstylesheet"),
+		intranetstylesheet => C4::Context->preference("intranetstylesheet"),
+		IntranetNav => C4::Context->preference("IntranetNav"),
+		);
 print $input->header(
 -type => guesstype($template->output),
 -cookie => $cookie
