@@ -24,6 +24,7 @@ use C4::Date;
 use MARC::Record;
 use C4::Suggestions;
 # use C4::Biblio;
+use Date::Calc qw(:all);
 
 use vars qw($VERSION @ISA @EXPORT);
 
@@ -60,7 +61,8 @@ orders, converting money to different currencies, and so forth.
 		&updaterecorder &newordernum
 		&getsupplierlistwithlateorders
 		&getlateorders
-
+		&getparcels &getparcelinformation
+		
 		&bookfunds &curconvert &getcurrencies &bookfundbreakdown
 		&updatecurrencies &getcurrency
 
@@ -85,7 +87,7 @@ get all basket informations in aqbasket for a given basket
 sub getbasket {
 	my ($basketno)=@_;
 	my $dbh=C4::Context->dbh;
-	my $sth=$dbh->prepare("select aqbasket.*,borrowers.firstname+' '+borrowers.surname as authorisedbyname from aqbasket left join borrowers on aqbasket.authorisedby=borrowers.borrowernumber where basketno=?");
+	my $sth=$dbh->prepare("select aqbasket.*,concat(borrowers.firstname,' ',borrowers.surname) as authorisedbyname from aqbasket left join borrowers on aqbasket.authorisedby=borrowers.borrowernumber where basketno=?");
 	$sth->execute($basketno);
 	return($sth->fetchrow_hashref);
 }
@@ -316,17 +318,23 @@ Also updates the book fund ID in the aqorderbreakdown table.
 =cut
 #'
 sub receiveorder {
-	my ($biblio,$ordnum,$quantrec,$user,$cost,$invoiceno,$freight,$rrp)=@_;
-	my $dbh = C4::Context->dbh;
-	my $sth=$dbh->prepare("update aqorders set quantityreceived=?,datereceived=now(),booksellerinvoicenumber=?,
+	my ($biblio,$ordnum,$quantrec,$user,$cost,$invoiceno,$datereceived,$freight,$rrp)=@_;
+    my $dbh = C4::Context->dbh;
+    unless ($datereceived) {
+        # if datereceived is not set, calculate today date
+        my ($year,$month,$day) = Today();
+        $datereceived = "$year-$month-$day";
+    }
+	my $sth=$dbh->prepare("update aqorders set quantityreceived=?,datereceived=?,booksellerinvoicenumber=?,
 											unitprice=?,freight=?,rrp=?
 							where biblionumber=? and ordernumber=?");
 	my $suggestionid = findsuggestion_from_biblionumber($dbh,$biblio);
 	if ($suggestionid) {
 		changestatus($suggestionid,'AVAILABLE','',$biblio);
 	}
-	$sth->execute($quantrec,$invoiceno,$cost,$freight,$rrp,$biblio,$ordnum);
-	$sth->finish;
+	$sth->execute($quantrec,$datereceived,$invoiceno,$cost,$freight,$rrp,$biblio,$ordnum);
+    $sth->finish;
+    return $datereceived;
 }
 
 =item updaterecorder
@@ -405,8 +413,7 @@ from aqorders
 left join aqbasket on aqbasket.basketno=aqorders.basketno 
 left join borrowers on aqbasket.authorisedby=borrowers.borrowernumber
 where booksellerid=? and (quantity > quantityreceived or
-quantityreceived is NULL) and datecancellationprinted is NULL ";
-		
+quantityreceived is NULL) and datecancellationprinted is NULL and (to_days(now())-to_days(closedate) < 180 or closedate is null)";
 	if (C4::Context->preference("IndependantBranches")) {
 		my $userenv = C4::Context->userenv;
 		if (($userenv)&&($userenv->{flags} != 1)){
@@ -496,8 +503,8 @@ sub getallorders {
   my ($supplierid)=@_;
   my $dbh = C4::Context->dbh;
   my @results = ();
-	my $strsth ="Select count(*),authorisedby,creationdate,aqbasket.basketno,
-closedate,surname,firstname,aqorders.title 
+	my $strsth ="Select authorisedby,creationdate,aqbasket.basketno,
+closedate,surname,firstname,aqorders.biblionumber,aqorders.title, aqorders.ordernumber, quantity, quantityreceived
 from aqorders 
 left join aqbasket on aqbasket.basketno=aqorders.basketno 
 left join borrowers on aqbasket.authorisedby=borrowers.borrowernumber
@@ -510,7 +517,7 @@ quantityreceived is NULL) and datecancellationprinted is NULL ";
 			$strsth .= " and (borrowers.branchcode = '".$userenv->{branch}."' or borrowers.branchcode ='')";
 		}
 	}
-	$strsth.=" group by basketno order by aqbasket.basketno";
+	$strsth.=" order by aqbasket.basketno";
 	my $sth=$dbh->prepare($strsth);
   $sth->execute($supplierid);
   while (my $data=$sth->fetchrow_hashref){
@@ -519,6 +526,48 @@ quantityreceived is NULL) and datecancellationprinted is NULL ";
   $sth->finish;
   return(scalar(@results),@results);
 }
+=item getparcelinformation
+
+  ($count, @results) = &getparcelinformation($booksellerid, $code, $date);
+
+Looks up all of the received items from the supplier with the given
+bookseller ID at the given date, for the given code. Ignores cancelled and completed orders.
+
+C<$count> is the number of elements in C<@results>. C<@results> is an
+array of references-to-hash. The keys of each element are fields from
+the aqorders, biblio, and biblioitems tables of the Koha database.
+
+C<@results> is sorted alphabetically by book title.
+
+=cut
+#'
+sub getparcelinformation {
+  #gets all orders from a certain supplier, orders them alphabetically
+  my ($supplierid,$code, $datereceived)=@_;
+  my $dbh = C4::Context->dbh;
+  my @results = ();
+  $code .='%' if $code; # add % if we search on a given code (otherwise, let him empty)
+	my $strsth ="Select authorisedby,creationdate,aqbasket.basketno,closedate,surname,firstname,aqorders.biblionumber,aqorders.title,aqorders.ordernumber, aqorders.quantity, aqorders.quantityreceived, aqorders.unitprice, aqorders.listprice, aqorders.rrp, aqorders.ecost from aqorders,aqbasket left join borrowers on aqbasket.authorisedby=borrowers.borrowernumber where aqbasket.basketno=aqorders.basketno and aqbasket.booksellerid=? and aqorders.booksellerinvoicenumber like  \"$code\" and aqorders.datereceived= \'$datereceived\'";
+		
+	if (C4::Context->preference("IndependantBranches")) {
+		my $userenv = C4::Context->userenv;
+		if (($userenv) &&($userenv->{flags} != 1)){
+			$strsth .= " and (borrowers.branchcode = '".$userenv->{branch}."' or borrowers.branchcode ='')";
+		}
+	}
+	$strsth.=" order by aqbasket.basketno";
+    ### parcelinformation : $strsth
+	my $sth=$dbh->prepare($strsth);
+  $sth->execute($supplierid);
+  while (my $data=$sth->fetchrow_hashref){
+    push(@results,$data);
+  }
+  my $count =scalar(@results);
+  ### countparcelbiblio: $count
+  $sth->finish;
+  
+  return(scalar(@results),@results);
+}
 =item getsupplierlistwithlateorders
 
   %results = &getsupplierlistwithlateorders;
@@ -618,7 +667,6 @@ sub getlateorders {
 		$strsth .= " AND borrowers.branchcode like \'".C4::Context->userenv->{branch}."\'" if (C4::Context->preference("IndependantBranches") && C4::Context->userenv->{flags}!=1);
 		$strsth .= " ORDER BY latesince,basketno,borrowers.branchcode, supplier";
 	}
-	warn "C4::Acquisition : getlateorders SQL:".$strsth;
 	my $sth = $dbh->prepare($strsth);
 	$sth->execute;
 	my @results;
@@ -698,11 +746,11 @@ sub ordersearch {
 	my @searchterms = ($id);
 	map { push(@searchterms,"$_%","% $_%") } @data;
 	push(@searchterms,$search,$search,$biblio);
-	my $sth=$dbh->prepare("Select biblio.*,biblioitems.*,aqorders.*,aqbasket.*,biblio.title from aqorders,biblioitems,biblio,aqbasket
-		where aqorders.biblioitemnumber = biblioitems.biblioitemnumber and
-		aqorders.basketno = aqbasket.basketno
-		and aqbasket.booksellerid = ?
-		and biblio.biblionumber=aqorders.biblionumber
+    my $sth=$dbh->prepare("SELECT biblio.*,biblioitems.*,aqorders.*,aqbasket.*,biblio.title from aqorders
+        LEFT JOIN aqbasket ON aqorders.basketno = aqbasket.basketno
+        LEFT JOIN biblioitems ON aqorders.biblioitemnumber = biblioitems.biblioitemnumber
+        LEFT JOIN biblio ON biblio.biblionumber=aqorders.biblionumber
+		where aqbasket.booksellerid = ?
 		and ((datecancellationprinted is NULL)
 		or (datecancellationprinted = '0000-00-00'))
 		and (("
@@ -748,8 +796,8 @@ sub histsearch {
 		$query .= " and biblio.title like ".$dbh->quote("%".$title."%") if $title;
 		$query .= " and biblio.author like ".$dbh->quote("%".$author."%") if $author;
 		$query .= " and name like ".$dbh->quote("%".$name."%") if $name;
-		$query .= " and creationdate >" .$dbh->quote($from_placed_on) if $from_placed_on;
-		$query .= " and creationdate<".$dbh->quote($to_placed_on) if $to_placed_on;
+		$query .= " and creationdate >=" .$dbh->quote($from_placed_on) if $from_placed_on;
+		$query .= " and creationdate=<".$dbh->quote($to_placed_on) if $to_placed_on;
 		if (C4::Context->preference("IndependantBranches")) {
 			my $userenv = C4::Context->userenv;
 			if (($userenv) &&($userenv->{flags} != 1)){
@@ -757,7 +805,6 @@ sub histsearch {
 			}
 		}
 		$query .=" order by booksellerid";
-		warn "query histearch: ".$query;
 		my $sth = $dbh->prepare($query);
 		$sth->execute;
 		my $cnt=1;
@@ -1033,7 +1080,6 @@ sub branches {
 		my $strsth ="Select * from branches ";
 		$strsth.= " WHERE branchcode = ".$dbh->quote(C4::Context->userenv->{branch});
 		$strsth.= " order by branchname";
-		warn "C4::Acquisition->branches : ".$strsth;
 		$sth=$dbh->prepare($strsth);
 	} else {
     	$sth = $dbh->prepare("Select * from branches order by branchname");
@@ -1073,17 +1119,17 @@ sub updatesup {
    contemail=?,contnotes=?,active=?,
    listprice=?, invoiceprice=?,gstreg=?, listincgst=?,
    invoiceincgst=?, specialty=?,discount=?,invoicedisc=?,
-   nocalc=?
+   nocalc=?, notes=?
    where id=?");
    $sth->execute($data->{'name'},$data->{'address1'},$data->{'address2'},
    $data->{'address3'},$data->{'address4'},$data->{'postal'},$data->{'phone'},
    $data->{'fax'},$data->{'url'},$data->{'contact'},$data->{'contpos'},
    $data->{'contphone'},$data->{'contfax'},$data->{'contaltphone'},
    $data->{'contemail'},
-   $data->{'contnote'},$data->{'active'},$data->{'listprice'},
+   $data->{'contnotes'},$data->{'active'},$data->{'listprice'},
    $data->{'invoiceprice'},$data->{'gstreg'},$data->{'listincgst'},
    $data->{'invoiceincgst'},$data->{'specialty'},$data->{'discount'},
-   $data->{'invoicedisc'},$data->{'nocalc'},$data->{'id'});
+   $data->{'invoicedisc'},$data->{'nocalc'},$data->{'notes'},$data->{'id'});
    $sth->finish;
 }
 
@@ -1115,6 +1161,42 @@ sub insertsup {
   return($data->{'id'});
 }
 
+=item getparcels
+
+  ($count, $results) = &getparcels($dbh, $bookseller, $order, $limit);
+
+get a lists of parcels
+Returns the count of parcels returned and a pointer on a hash list containing parcel informations as such :
+		Creation date
+		Last operation
+		Number of biblio
+		Number of items
+		
+
+=cut
+#'
+sub getparcels {
+  my ($bookseller, $order, $code,$datefrom,$dateto, $limit)=@_;
+	my $dbh = C4::Context->dbh;
+	my $strsth = "SELECT aqorders.booksellerinvoicenumber, datereceived, count(DISTINCT biblionumber) as biblio, sum(quantity) as itemsexpected, sum(quantityreceived) as itemsreceived from aqorders, aqbasket where aqbasket.basketno = aqorders.basketno and aqbasket.booksellerid = $bookseller and datereceived is not null ";
+	$strsth .= "and aqorders.booksellerinvoicenumber like \"$code%\" " if ($code);
+	$strsth .= "and datereceived >=".$dbh->quote($datefrom)." " if ($datefrom);
+	$strsth .= "and datereceived <=".$dbh->quote($dateto)." " if ($dateto);
+	$strsth .= "group by aqorders.booksellerinvoicenumber,datereceived ";
+	$strsth .= "order by $order " if ($order);
+	$strsth .= " LIMIT 0,$limit" if ($limit);
+	my $sth=$dbh->prepare($strsth);
+###	getparcels:  $strsth
+	$sth->execute;
+	my @results;
+	while (my $data2=$sth->fetchrow_hashref) {
+		push @results, $data2;
+	}
+	
+   $sth->finish;
+   return(scalar(@results), @results);
+}
+
 END { }       # module clean-up code here (global destructor)
 
 1;
