@@ -196,7 +196,34 @@ sub get_template_and_user {
 			$template->param(CAN_user_tools => 1); }
 		
 	}
+	if  ($in->{'type'} eq "intranet") {
+        $template->param(
+                        intranetcolorstylesheet => C4::Context->preference("intranetcolorstylesheet"),  
+                        intranetstylesheet => C4::Context->preference("intranetstylesheet"),
+                        IntranetNav => C4::Context->preference("IntranetNav"),
+
+        );
+
+	}
+        else {
 	$template->param(
+				suggestion => C4::Context->preference("suggestion"),
+				virtualshelves => C4::Context->preference("virtualshelves"),
+				OpacNav => C4::Context->preference("OpacNav"),
+				opacheader	=> C4::Context->preference("opacheader"),
+				opaccredits => C4::Context->preference("opaccredits"),
+				opacsmallimage => C4::Context->preference("opacsmallimage"),
+				opaclayoutstylesheet => C4::Context->preference("opaclayoutstylesheet"),
+				opaccolorstylesheet => C4::Context->preference("opaccolorstylesheet"),
+				opaclanguagesdisplay => C4::Context->preference("opaclanguagesdisplay"),
+				TemplateEncoding => C4::Context->preference("TemplateEncoding"),
+				opacuserlogin => C4::Context->preference("opacuserlogin"),
+				opacbookbag => C4::Context->preference("opacbookbag"),
+		);
+	}
+	$template->param(
+				TemplateEncoding => C4::Context->preference("TemplateEncoding"),
+				AmazonContent => C4::Context->preference("AmazonContent"),
 			     LibraryName => C4::Context->preference("LibraryName"),
 		);
 	return ($template, $borrowernumber, $cookie);
@@ -303,6 +330,7 @@ sub checkauth {
 					$hash{firstname},
 					$hash{surname},
 					$hash{branch},
+					$hash{branchname},
 					$hash{flags},
 					$hash{emailaddress},
 				);
@@ -388,19 +416,19 @@ sub checkauth {
 					C4::Context->_unset_userenv($sessionID);
 			}
 			if ($return == 1){
-				my ($bornum,$firstname,$surname,$userflags,$branchcode,$emailaddress);
-				my $sth=$dbh->prepare("select borrowernumber,firstname,surname,flags,branchcode,emailaddress from borrowers where userid=?");
+				my ($bornum,$firstname,$surname,$userflags,$branchcode,$branchname,$emailaddress);
+				my $sth=$dbh->prepare("select borrowernumber,firstname,surname,flags,borrowers.branchcode,branchname,emailaddress from borrowers left join branches on borrowers.branchcode=branches.branchcode where userid=?");
 				$sth->execute($userid);
-				($bornum,$firstname,$surname,$userflags,$branchcode,$emailaddress) = $sth->fetchrow if ($sth->rows);
+				($bornum,$firstname,$surname,$userflags,$branchcode,$branchname, $emailaddress) = $sth->fetchrow if ($sth->rows);
 # 				warn "$cardnumber,$bornum,$userid,$firstname,$surname,$userflags,$branchcode,$emailaddress";
 				unless ($sth->rows){
-					my $sth=$dbh->prepare("select borrowernumber,firstname,surname,flags,branchcode,emailaddress from borrowers where cardnumber=?");
+					my $sth=$dbh->prepare("select borrowernumber,firstname,surname,flags,borrowers.branchcode,branchname,emailaddress from borrowers left join branches on borrowers.branchcode=branches.branchcode where cardnumber=?");
 					$sth->execute($cardnumber);
-					($bornum,$firstname,$surname,$userflags,$branchcode,$emailaddress) = $sth->fetchrow if ($sth->rows);
+					($bornum,$firstname,$surname,$userflags,$branchcode, $branchname,$emailaddress) = $sth->fetchrow if ($sth->rows);
 # 					warn "$cardnumber,$bornum,$userid,$firstname,$surname,$userflags,$branchcode,$emailaddress";
 					unless ($sth->rows){
 						$sth->execute($userid);
-						($bornum,$firstname,$surname,$userflags,$branchcode,$emailaddress) = $sth->fetchrow if ($sth->rows);
+						($bornum,$firstname,$surname,$userflags,$branchcode, $branchname, $emailaddress) = $sth->fetchrow if ($sth->rows);
 					}
 # 					warn "$cardnumber,$bornum,$userid,$firstname,$surname,$userflags,$branchcode,$emailaddress";
 				}
@@ -411,6 +439,7 @@ sub checkauth {
 					$firstname,
 					$surname,
 					$branchcode,
+					$branchname, 
 					$userflags,
 					$emailaddress,
 				);
@@ -425,7 +454,7 @@ sub checkauth {
 					C4::Context->config('user'),
 					C4::Context->config('user'),
 					C4::Context->config('user'),
-					"",1,C4::Context->preference('KohaAdminEmailAddress')
+					"","",1,C4::Context->preference('KohaAdminEmailAddress')
 				);
 				$envcookie=$query->cookie(-name => 'userenv',
 						-value => $hash,
@@ -463,7 +492,14 @@ sub checkauth {
 	}
 
 	my $template = gettemplate($template_name, $type,$query);
-	$template->param(INPUTS => \@inputs);
+	$template->param(INPUTS => \@inputs,
+			intranetcolorstylesheet => C4::Context->preference("intranetcolorstylesheet"),
+			intranetstylesheet => C4::Context->preference("intranetstylesheet"),
+			IntranetNav => C4::Context->preference("IntranetNav"),
+			opacnav => C4::Context->preference("OpacNav"),
+			TemplateEncoding => C4::Context->preference("TemplateEncoding"),
+
+			);
 	$template->param(loginprompt => 1) unless $info{'nopermission'};
 
 	my $self_url = $query->url(-absolute => 1);
