@@ -47,11 +47,16 @@ $VERSION = 0.01;
 	&authoritysearch
 	
 	&MARCmodsubfield
-	&AUTHhtml2marc
+	&AUTHhtml2marc &AUTHhtml2xml
 	&AUTHaddword
 	&MARCaddword &MARCdelword
 	&char_decode
 	&FindDuplicate
+	&BuildSummary
+	&BuildUnimarcHierarchies
+	&BuildUnimarcHierarchy
+    &AUTHsavetrees
+    &AUTHgetheader
  );
 
 sub authoritysearch {
@@ -66,23 +71,36 @@ sub authoritysearch {
 	# the authtypecode. Then, search on $a of this tag_to_report
 	# also store main entry MARC tag, to extract it at end of search
 	my $mainentrytag;
-	my $sth = $dbh->prepare("select auth_tag_to_report from auth_types where authtypecode=?");
-	$sth->execute($authtypecode);
+	my $sth;
+    if ($authtypecode){
+      $sth= $dbh->prepare('select auth_tag_to_report from auth_types where authtypecode=?');
+      $sth->execute($authtypecode);
+    }else{
+      $sth= $dbh->prepare('select auth_tag_to_report from auth_types');
+      $sth->execute;
+    }
 	my ($tag_to_report) = $sth->fetchrow;
 	$mainentrytag = $tag_to_report;
-	for (my $i=0;$i<$#{$tags};$i++) {
-		if (@$tags[$i] eq "mainentry") {
-			@$tags[$i] = $tag_to_report."a";
-		}
-	}
-
-	# "Normal" statements
 	# quote marc fields/subfields
 	for (my $i=0;$i<=$#{$tags};$i++) {
 		if (@$tags[$i]) {
 			@$tags[$i] = $dbh->quote(@$tags[$i]);
 		}
 	}
+	for (my $i=0;$i<$#{$tags};$i++) {
+		if (@$tags[$i] eq "'mainentry'") {
+            @$tags[$i]="'";
+            foreach ('a'..'z') {
+                @$tags[$i] .= $tag_to_report.$_."','";
+            }
+            chop @$tags[$i];
+            chop @$tags[$i];
+		}
+		if (@$tags[$i] eq '\'$a\'') {
+			@$tags[$i] = "'".$tag_to_report.'a\'';
+		}
+	}
+
 	my @normal_tags = ();
 	my @normal_and_or = ();
 	my @normal_operator = ();
@@ -123,14 +141,25 @@ sub authoritysearch {
 
 	my $sth;
 
-	if ($sql_where2) {
-		$sth = $dbh->prepare("select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and auth_header.authtypecode=? and $sql_where2 and ($sql_where1)");
-		warn "Q2 : select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and auth_header.authtypecode=? and $sql_where2 and ($sql_where1)";
-	} else {
-		$sth = $dbh->prepare("select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and auth_header.authtypecode=? and $sql_where1");
-		warn "Q : select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and auth_header.authtypecode=? and $sql_where1";
-	}
-	$sth->execute($authtypecode);
+	if ($authtypecode){
+      if ($sql_where2) {
+          $sth = $dbh->prepare("select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and auth_header.authtypecode=? and $sql_where2 and ($sql_where1)");
+          warn "Q2 : select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and auth_header.authtypecode=? and $sql_where2 and ($sql_where1)";
+      } else {
+          $sth = $dbh->prepare("select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and auth_header.authtypecode=? and $sql_where1");
+          warn "Q : select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and auth_header.authtypecode=? and $sql_where1";
+      }
+      $sth->execute("$authtypecode");
+    } else {
+      if ($sql_where2) {
+          $sth = $dbh->prepare("select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and $sql_where2 and ($sql_where1)");
+          warn "Q2 : select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid  and $sql_where2 and ($sql_where1)";
+      } else {
+          $sth = $dbh->prepare("select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and $sql_where1");
+          warn "Q : select distinct m1.authid from auth_header,$sql_tables where  m1.authid=auth_header.authid and $sql_where1";
+      }
+      $sth->execute;
+    }
 	my @result = ();
 	while (my ($authid) = $sth->fetchrow) {
 			push @result,$authid;
@@ -146,28 +175,16 @@ sub authoritysearch {
 		# get MARC::Record of the authority
 		my $record = AUTHgetauthority($dbh,$result[$counter]);
 		# then build the summary
+		#FIXME: all of this should be moved to the template eventually
 		my $authtypecode = AUTHfind_authtypecode($dbh,$result[$counter]);
 		my $authref = getauthtype($authtypecode);
+		my $authtype =$authref->{authtypetext};
 		my $summary = $authref->{summary};
-		my @fields = $record->fields();
-		foreach my $field (@fields) {
-			my $tag = $field->tag();
-			if ($tag<10) {
-			} else {
-				my @subf = $field->subfields;
-				for my $i (0..$#subf) {
-					my $subfieldcode = $subf[$i][0];
-					my $subfieldvalue = $subf[$i][1];
-					my $tagsubf = $tag.$subfieldcode;
-					$summary =~ s/\[(.?.?.?.?)$tagsubf(.*?)]/$1$subfieldvalue$2\[$1$tagsubf$2]/g;
-				}
-			}
-		}
-		$summary =~ s/\[(.*?)]//g;
-		$summary =~ s/\n/<br>/g;
-
+		my $query_auth_tag = "SELECT auth_tag_to_report FROM auth_types WHERE authtypecode=?";
+		my $sth = $dbh->prepare($query_auth_tag);
+		$sth->execute($authtypecode);
+		my $auth_tag_to_report = $sth->fetchrow;
 		# find biblio MARC field using this authtypecode (to jump to biblio)
-		my $authtypecode = AUTHfind_authtypecode($dbh,$result[$counter]);
 		my $sth = $dbh->prepare("select distinct tagfield from marc_subfield_structure where authtypecode=?");
 		$sth->execute($authtypecode);
 		my $tags_using_authtype;
@@ -176,10 +193,112 @@ sub authoritysearch {
 			$tags_using_authtype.= $tagfield."9,";
 		}
 		chop $tags_using_authtype;
-		
+		my $reported_tag;
+		# if the library has a summary defined, use it. Otherwise, build a standard one
+		if ($summary) {
+			my @fields = $record->fields();
+            		$reported_tag = '$9'.$result[$counter];
+			foreach my $field (@fields) {
+				my $tag = $field->tag();
+				my $tagvalue = $field->as_string();
+				$summary =~ s/\[(.?.?.?.?)$tag\*(.*?)]/$1$tagvalue$2\[$1$tag$2]/g;
+				if ($tag<10) {
+					if ($tag eq '001') {
+					$reported_tag.='$3'.$field->data();
+					}
+				} else {
+					my @subf = $field->subfields;
+					for my $i (0..$#subf) {
+						my $subfieldcode = $subf[$i][0];
+						my $subfieldvalue = $subf[$i][1];
+						my $tagsubf = $tag.$subfieldcode;
+						$summary =~ s/\[(.?.?.?.?)$tagsubf(.*?)]/$1$subfieldvalue$2\[$1$tagsubf$2]/g;
+						if ($tag eq $auth_tag_to_report) {
+			    				$reported_tag.='$'.$subfieldcode.$subfieldvalue;
+						}
+
+					}
+				}
+			}
+			$summary =~ s/\[(.*?)]//g;
+			$summary =~ s/\n/<br>/g;
+		} else {
+			my $heading; 
+			my $altheading;
+			my $seeheading;
+			my $see;
+			my @fields = $record->fields();
+			if (C4::Context->preference('marcflavour') eq 'UNIMARC') {
+			# construct UNIMARC summary, that is quite different from MARC21 one
+				# accepted form
+				foreach my $field ($record->field('2..')) {
+					$heading.= $field->as_string();
+				}
+				# rejected form(s)
+				foreach my $field ($record->field('4..')) {
+					$summary.= "&nbsp;&nbsp;&nbsp;<i>".$field->as_string()."</i><br/>";
+					$summary.= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>see:</i> ".$heading."<br/>";
+				}
+				# see :
+				foreach my $field ($record->field('5..')) {
+					$summary.= "&nbsp;&nbsp;&nbsp;<i>".$field->as_string()."</i><br/>";
+					$summary.= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>see:</i> ".$heading."<br/>";
+				}
+				# // form
+				foreach my $field ($record->field('7..')) {
+					$seeheading.= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>see also:</i> ".$field->as_string()."<br />";	
+					$altheading.= "&nbsp;&nbsp;&nbsp;".$field->as_string()."<br />";
+					$altheading.= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>see also:</i> ".$heading."<br />";
+				}
+				$summary = "<b>".$heading."</b><br />".$seeheading.$altheading.$summary;	
+			} else {
+			# construct MARC21 summary
+				foreach my $field ($record->field('1..')) {
+					if ($record->field('100')) {
+						$heading.= $field->as_string('abcdefghjklmnopqrstvxyz68');
+					} elsif ($record->field('110')) {
+	                                        $heading.= $field->as_string('abcdefghklmnoprstvxyz68');
+					} elsif ($record->field('111')) {
+	                                        $heading.= $field->as_string('acdefghklnpqstvxyz68');
+					} elsif ($record->field('130')) {
+	                                        $heading.= $field->as_string('adfghklmnoprstvxyz68');
+					} elsif ($record->field('148')) {
+	                                        $heading.= $field->as_string('abvxyz68');
+					} elsif ($record->field('150')) {
+						$heading.= $field->as_string('abvxyz68');	
+					} elsif ($record->field('151')) {
+	                                        $heading.= $field->as_string('avxyz68');
+					} elsif ($record->field('155')) {
+	                                        $heading.= $field->as_string('abvxyz68');
+					} elsif ($record->field('180')) {
+	                                        $heading.= $field->as_string('vxyz68');
+					} elsif ($record->field('181')) {
+	                                        $heading.= $field->as_string('vxyz68');
+					} elsif ($record->field('182')) {
+	                                        $heading.= $field->as_string('vxyz68');
+					} elsif ($record->field('185')) {
+	                                        $heading.= $field->as_string('vxyz68');
+					} else {
+						$heading.= $field->as_string();
+					}
+				} #See From
+				foreach my $field ($record->field('4..')) {
+					$summary.= "&nbsp;&nbsp;&nbsp;".$field->as_string()."<br />";
+					$summary.= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>see:</i> ".$heading."<br />";	
+				} #See Also
+				foreach my $field ($record->field('5..')) {
+					$seeheading.= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>see also:</i> ".$field->as_string()."<br />";	
+					$altheading.= "&nbsp;&nbsp;&nbsp;<b>".$field->as_string()."</b><br />";
+					$altheading.= "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>see also:</i> ".$heading."<br />";
+				}
+				$summary = "<b>".$heading."</b>".$seeheading.$altheading.$summary;
+			}
+		}
 		# then add a line for the template loop
 		my %newline;
 		$newline{summary} = $summary;
+		$newline{authtype} = $authtype;
+		$newline{reported_tag} = $reported_tag;
 		$newline{authid} = $result[$counter];
 		$newline{used} = &AUTHcount_usage($result[$counter]);
 		$newline{biblio_fields} = $tags_using_authtype;
@@ -217,14 +336,14 @@ sub create_request {
 			if ($nb_active==1) {
 				if (@$operator[$i] eq "start") {
 					$sql_tables .= "auth_subfield_table as m$nb_table,";
-					$sql_where1 .= "(m1.subfieldvalue like ".$dbh->quote("@$value[$i]%");
+					$sql_where1 .= "(m1.subfieldvalue like ".$dbh->quote("@$value[$i]");
 					if (@$tags[$i]) {
-						$sql_where1 .=" and m1.tag+m1.subfieldcode in (@$tags[$i])";
+						$sql_where1 .=" and concat(m1.tag,m1.subfieldcode) in (@$tags[$i])";
 					}
 					$sql_where1.=")";
 				} elsif (@$operator[$i] eq "contains") {	
 				$sql_tables .= "auth_word as m$nb_table,";
-					$sql_where1 .= "(m1.word  like ".$dbh->quote("@$value[$i]%");
+					$sql_where1 .= "(m1.word  like ".$dbh->quote("@$value[$i]");
 					if (@$tags[$i]) {
 						 $sql_where1 .=" and m1.tagsubfield in (@$tags[$i])";
 					}
@@ -234,7 +353,7 @@ sub create_request {
 					$sql_tables .= "auth_subfield_table as m$nb_table,";
 					$sql_where1 .= "(m1.subfieldvalue @$operator[$i] ".$dbh->quote("@$value[$i]");
 					if (@$tags[$i]) {
-						 $sql_where1 .=" and m1.tag+m1.subfieldcode in (@$tags[$i])";
+						 $sql_where1 .=" and concat(m1.tag,m1.subfieldcode) in (@$tags[$i])";
 					}
 					$sql_where1.=")";
 				}
@@ -242,9 +361,9 @@ sub create_request {
 				if (@$operator[$i] eq "start") {
 					$nb_table++;
 					$sql_tables .= "auth_subfield_table as m$nb_table,";
-					$sql_where1 .= "@$and_or[$i] (m$nb_table.subfieldvalue like ".$dbh->quote("@$value[$i]%");
+					$sql_where1 .= "@$and_or[$i] (m$nb_table.subfieldvalue like ".$dbh->quote("@$value[$i]");
 					if (@$tags[$i]) {
-					 	$sql_where1 .=" and m$nb_table.tag+m$nb_table.subfieldcode in (@$tags[$i])";
+					 	$sql_where1 .=" and concat(m$nb_table.tag,m$nb_table.subfieldcode) in (@$tags[$i])";
 					}
 					$sql_where1.=")";
 					$sql_where2 .= "m1.authid=m$nb_table.authid and ";
@@ -252,16 +371,16 @@ sub create_request {
 					if (@$and_or[$i] eq 'and') {
 						$nb_table++;
 						$sql_tables .= "auth_word as m$nb_table,";
-						$sql_where1 .= "@$and_or[$i] (m$nb_table.word like ".$dbh->quote("@$value[$i]%");
+						$sql_where1 .= "@$and_or[$i] (m$nb_table.word like ".$dbh->quote("@$value[$i]");
 						if (@$tags[$i]) {
 							$sql_where1 .=" and m$nb_table.tagsubfield in(@$tags[$i])";
 						}
 						$sql_where1.=")";
 						$sql_where2 .= "m1.authid=m$nb_table.authid and ";
 					} else {
-						$sql_where1 .= "@$and_or[$i] (m$nb_table.word like ".$dbh->quote("@$value[$i]%");
+						$sql_where1 .= "@$and_or[$i] (m$nb_table.word like ".$dbh->quote("@$value[$i]");
 						if (@$tags[$i]) {
-							$sql_where1 .="  and m$nb_table.tag+m$nb_table.subfieldid in (@$tags[$i])";
+							$sql_where1 .="  and concat(m$nb_table.tag,m$nb_table.subfieldid) in (@$tags[$i])";
 						}
 						$sql_where1.=")";
 						$sql_where2 .= "m1.authid=m$nb_table.authid and ";
@@ -271,7 +390,7 @@ sub create_request {
 					$sql_tables .= "auth_subfield_table as m$nb_table,";
 					$sql_where1 .= "@$and_or[$i] (m$nb_table.subfieldvalue @$operator[$i] ".$dbh->quote(@$value[$i]);
 					if (@$tags[$i]) {
-					 	$sql_where1 .="  and m$nb_table.tag+m$nb_table.subfieldcode in (@$tags[$i])";
+					 	$sql_where1 .="  and concat(m$nb_table.tag,m$nb_table.subfieldcode) in (@$tags[$i])";
 					}
 					$sql_where2 .= "m1.authid=m$nb_table.authid and ";
 					$sql_where1.=")";
@@ -409,15 +528,22 @@ sub AUTHaddauthority {
 	my @fields=$record->fields();
 # adding main table, and retrieving authid
 # if authid is sent, then it's not a true add, it's only a re-add, after a delete (ie, a mod)
-# if authid empty => true add, find a new authid number
-	unless ($authid) {
+#  In fact, it could still be a true add, in the case of a bulkauthimort for instance with previously
+#  existing authids in the records. I've adjusted below to account for this instance --JF.
+	if ($authid) {
 		$dbh->do("lock tables auth_header WRITE,auth_subfield_table WRITE, auth_word WRITE, stopwords READ");
-		my $sth=$dbh->prepare("insert into auth_header (datecreated,authtypecode) values (now(),?)");
-		$sth->execute($authtypecode);
-		$sth=$dbh->prepare("select max(authid) from auth_header");
-		$sth->execute;
-		($authid)=$sth->fetchrow;
+		my $sth=$dbh->prepare("insert into auth_header (authid,datecreated,authtypecode) values (?,now(),?)");
+		$sth->execute($authid,$authtypecode);
 		$sth->finish;
+# if authid empty => true add, find a new authid number
+	} else {
+        $dbh->do("lock tables auth_header WRITE,auth_subfield_table WRITE, auth_word WRITE, stopwords READ");
+        my $sth=$dbh->prepare("insert into auth_header (datecreated,authtypecode) values (now(),?)");
+        $sth->execute($authtypecode);
+        $sth=$dbh->prepare("select max(authid) from auth_header");
+        $sth->execute;
+        ($authid)=$sth->fetchrow;
+        $sth->finish;
 	}
 	my $fieldcount=0;
 	# now, add subfields...
@@ -660,6 +786,63 @@ sub AUTHdelsubfield {
 			");
 }
 
+sub AUTHhtml2xml {
+        my ($tags,$subfields,$values,$indicator,$ind_tag) = @_;
+        use MARC::File::XML;
+        my $xml= MARC::File::XML::header();
+        my $prevvalue;
+        my $prevtag=-1;
+        my $first=1;
+        my $j = -1;
+        for (my $i=0;$i<=@$tags;$i++){
+
+            if ((@$tags[$i] ne $prevtag)){
+                $j++ unless (@$tags[$i] eq "");
+
+                if (!$first){
+                    $xml.="</datafield>\n";
+                    $first=1;
+                }
+                else {
+                    if (@$values[$i] ne "") {
+                    # leader
+                    if (@$tags[$i] eq "000") {
+                        $xml.="<leader>@$values[$i]</leader>\n";
+                        $first=1;
+                        # rest of the fixed fields
+                    } elsif (@$tags[$i] < 10) {
+                        $xml.="<controlfield tag=\"@$tags[$i]\">@$values[$i]</controlfield>\n";
+                        $first=1;
+                    }
+                    else {
+                        my $ind1 = substr(@$indicator[$j],0,1);
+                        my $ind2 = substr(@$indicator[$j],1,1);
+                        $xml.="<datafield tag=\"@$tags[$i]\" ind1=\"$ind1\" ind2=\"$ind2\">\n";
+                        $xml.="<subfield code=\"@$subfields[$i]\">@$values[$i]</subfield>\n";
+                        $first=0;
+                    }
+                    }
+                }
+            } else {
+                if (@$values[$i] eq "") {
+                }
+                else {
+                if ($first){
+                my $ind1 = substr(@$indicator[$j],0,1);
+                my $ind2 = substr(@$indicator[$j],1,1);
+                $xml.="<datafield tag=\"@$tags[$i]\" ind1=\"$ind1\" ind2=\"$ind2\">\n";
+                $first=0;
+                }
+                    $xml.="<subfield code=\"@$subfields[$i]\">@$values[$i]</subfield>\n";
+
+                }
+            }
+            $prevtag = @$tags[$i];
+        }
+        $xml.= MARC::File::XML::footer();
+#         warn $xml;
+        return $xml
+}
 sub AUTHhtml2marc {
 	my ($dbh,$rtags,$rsubfields,$rvalues,%indicators) = @_;
 	my $prevtag = -1;
@@ -669,23 +852,39 @@ sub AUTHhtml2marc {
 	my $field; # if tag >=10
 	for (my $i=0; $i< @$rtags; $i++) {
 		# rebuild MARC::Record
+# 			warn "0=>".@$rtags[$i].@$rsubfields[$i]." = ".@$rvalues[$i].": ";
 		if (@$rtags[$i] ne $prevtag) {
 			if ($prevtag < 10) {
 				if ($prevvalue) {
-					$record->add_fields((sprintf "%03s",$prevtag),$prevvalue);
+					if (($prevtag ne '000') && ($prevvalue ne "")) {
+						$record->add_fields((sprintf "%03s",$prevtag),$prevvalue);
+					} elsif ($prevvalue ne ""){
+						$record->leader($prevvalue);
+					}
 				}
 			} else {
-				if ($field) {
+				if (($field) && ($field ne "")) {
 					$record->add_fields($field);
 				}
 			}
 			$indicators{@$rtags[$i]}.='  ';
+		        # skip blank tags, I hope this works 
+		        if (@$rtags[$i] eq ''){
+			    $prevtag = @$rtags[$i];
+			    undef $field;
+			    next;
+			}
 			if (@$rtags[$i] <10) {
 				$prevvalue= @$rvalues[$i];
 				undef $field;
 			} else {
 				undef $prevvalue;
+				if (@$rvalues[$i] eq "") {
+				undef $field;
+				} else {
 				$field = MARC::Field->new( (sprintf "%03s",@$rtags[$i]), substr($indicators{@$rtags[$i]},0,1),substr($indicators{@$rtags[$i]},1,1), @$rsubfields[$i] => @$rvalues[$i]);
+				}
+# 			warn "1=>".@$rtags[$i].@$rsubfields[$i]." = ".@$rvalues[$i].": ".$field->as_formatted;
 			}
 			$prevtag = @$rtags[$i];
 		} else {
@@ -693,14 +892,21 @@ sub AUTHhtml2marc {
 				$prevvalue=@$rvalues[$i];
 			} else {
 				if (length(@$rvalues[$i])>0) {
-					$field->add_subfields(@$rsubfields[$i] => @$rvalues[$i]);
+					if ($field) {
+						$field->add_subfields(@$rsubfields[$i] => @$rvalues[$i]);
+					} else {
+					$field = MARC::Field->new( (sprintf "%03s",@$rtags[$i]), substr($indicators{@$rtags[$i]},0,1),substr($indicators{@$rtags[$i]},1,1), @$rsubfields[$i] => @$rvalues[$i]);
+					}
+# 			warn "2=>".@$rtags[$i].@$rsubfields[$i]." = ".@$rvalues[$i].": ".$field->as_formatted;
 				}
 			}
 			$prevtag= @$rtags[$i];
 		}
 	}
+	#}
 	# the last has not been included inside the loop... do it now !
-	$record->add_fields($field) if $field;
+	$record->add_fields($field) if (($field) && $field ne "");
+ 	#warn "HTML2MARC=".$record->as_formatted;
 	return $record;
 }
 
@@ -875,7 +1081,7 @@ sub nsb_clean {
 
 sub FindDuplicate {
 	my ($record,$authtypecode)=@_;
-	warn "IN for ".$record->as_formatted;
+# 	warn "IN for ".$record->as_formatted;
 	my $dbh = C4::Context->dbh;
 
 #	warn "".$record->as_formatted;
@@ -891,7 +1097,7 @@ sub FindDuplicate {
 	my @subfield = split /\[/,  $taglist;
 	my $max = @subfield;
 	for (my $i=1; $i<$max;$i++){
-		warn " ".$subfield[$i];
+# 		warn " ".$subfield[$i];
 		$subfield[$i]=substr($subfield[$i],3,1);
 # 		warn " ".$subfield[$i];
 	}
@@ -902,7 +1108,7 @@ sub FindDuplicate {
 #		warn " field $auth_tag_to_report exists";
 		while (my ($tag,$subfield) = $sth->fetchrow){
 			if ($record->field($tag)->subfield($subfield)) {
-				warn "tag :".$tag." subfield: $subfield value : ".$record->field($tag)->subfield($subfield);
+# 				warn "tag :".$tag." subfield: $subfield value : ".$record->field($tag)->subfield($subfield);
 				push @tags, $tag.$subfield;
 #				warn "'".$tag.$subfield."' value :". $record->field($tag)->subfield($subfield);
 				push @and_or, "and";
@@ -916,13 +1122,204 @@ sub FindDuplicate {
 	my ($finalresult,$nbresult) = authoritysearch($dbh,\@tags,\@and_or,\@excluding,\@operator,\@value,0,10,$authtypecode);
 	# there is at least 1 result => return the 1st one
 	if ($nbresult) {
-		warn "XXXXX $nbresult => ".@$finalresult[0]->{authid},@$finalresult[0]->{summary};
+# 		warn "XXXXX $nbresult => ".@$finalresult[0]->{authid},@$finalresult[0]->{summary};
 		return @$finalresult[0]->{authid},@$finalresult[0]->{summary};
 	}
 	# no result, returns nothing
 	return;
 }
 
+sub BuildSummary{
+	my $record = shift @_;
+	my $summary = shift @_;
+    ##TODO : use langages from authorised_values
+    ## AND Thesaurii from auth_types
+	my %language;
+	$language{'fre'}="Franais";
+	$language{'eng'}="Anglais";
+	$language{'ger'}="Allemand";
+	$language{'ita'}="Italien";
+	$language{'spa'}="Espagnol";
+	my %thesaurus;
+	$thesaurus{'1'}="Peuples";
+	$thesaurus{'2'}="Anthroponymes";
+	$thesaurus{'3'}="Oeuvres";
+	$thesaurus{'4'}="Chronologie";
+	$thesaurus{'5'}="Lieux";
+	$thesaurus{'6'}="Sujets";
+	#thesaurus a remplir
+	if ($summary) {
+		my @fields = $record->fields();
+		foreach my $field (@fields) {
+			my $tag = $field->tag();
+			my $tagvalue = $field->as_string();
+			$summary =~ s/\[(.?.?.?.?)$tag\*(.*?)]/$1$tagvalue$2\[$1$tag$2]/g;
+			if ($tag<10) {
+			} else {
+				my @subf = $field->subfields;
+				for my $i (0..$#subf) {
+					my $subfieldcode = $subf[$i][0];
+					my $subfieldvalue = $subf[$i][1];
+					my $tagsubf = $tag.$subfieldcode;
+					$summary =~ s/\[(.?.?.?.?)$tagsubf(.*?)]/$1$subfieldvalue$2\[$1$tagsubf$2]/g;
+				}
+			}
+		}
+		$summary =~ s/\[(.*?)]//g;
+		$summary =~ s/\n/<br>/g;
+	} else {
+		my $heading; 
+		my $authid; 
+		my $altheading;
+		my $seealso;
+		my $broaderterms;
+		my $narrowerterms;
+		my $see;
+		my $seeheading;
+        my $notes;
+		my @fields = $record->fields();
+		if (C4::Context->preference('marcflavour') eq 'UNIMARC') {
+		# construct UNIMARC summary, that is quite different from MARC21 one
+			# accepted form
+			foreach my $field ($record->field('2..')) {
+				$heading.= $field->subfield('a');
+                $authid=$field->subfield('3');
+			}
+			# rejected form(s)
+			foreach my $field ($record->field('3..')) {
+			  $notes.= '<span class="note">'.$field->subfield('a')."</span>\n";
+			}
+			foreach my $field ($record->field('4..')) {
+			  my $thesaurus = "thes. : ".$thesaurus{"$field->subfield('2')"}." : " if ($field->subfield('2'));
+			  $see.= '<span class="UF">'.$thesaurus.$field->subfield('a')."</span> -- \n";
+			}
+			# see :
+			foreach my $field ($record->field('5..')) {
+		        
+				if (($field->subfield('5')) && ($field->subfield('a')) && ($field->subfield('5') eq 'g')) {
+				  $broaderterms.= '<span class="BT"> <a href="detail.pl?authid='.$field->subfield('3').'">'.$field->subfield('a')."</a></span> -- \n";
+				} elsif (($field->subfield('5')) && ($field->subfield('a')) && ($field->subfield('5') eq 'h')){
+				  $narrowerterms.= '<span class="NT"><a href="detail.pl?authid='.$field->subfield('3').'">'.$field->subfield('a')."</a></span> -- \n";
+				} elsif ($field->subfield('a')) {
+				  $seealso.= '<span class="RT"><a href="detail.pl?authid='.$field->subfield('3').'">'.$field->subfield('a')."</a></span> -- \n";
+				}
+			}
+			# // form
+			foreach my $field ($record->field('7..')) {
+				my $lang = substr($field->subfield('8'),3,3);
+				$seeheading.= '<span class="langue"> En '.$language{$lang}.' : </span><span class="OT"> '.$field->subfield('a')."</span><br />\n";	
+			}
+            $broaderterms =~s/-- \n$//;
+            $narrowerterms =~s/-- \n$//;
+            $seealso =~s/-- \n$//;
+            $see =~s/-- \n$//;
+			$summary = "<b><a href=\"detail.pl?authid=$authid\">".$heading."</a></b><br />".($notes?"$notes <br />":"");
+			$summary.= '<p><div class="label">TG : '.$broaderterms.'</div></p>' if ($broaderterms);
+			$summary.= '<p><div class="label">TS : '.$narrowerterms.'</div></p>' if ($narrowerterms);
+			$summary.= '<p><div class="label">TA : '.$seealso.'</div></p>' if ($seealso);
+			$summary.= '<p><div class="label">EP : '.$see.'</div></p>' if ($see);
+			$summary.= '<p><div class="label">'.$seeheading.'</div></p>' if ($seeheading);
+		}
+	}
+	return $summary;
+}
+
+sub BuildUnimarcHierarchies{
+  my $authid = shift @_;
+#   warn "authid : $authid";
+  my $force = shift @_;
+  my @globalresult;
+  my $dbh=C4::Context->dbh;
+  my $hierarchies;
+  my $data = AUTHgetheader($dbh,$authid);
+  
+  if ($data->{'authtrees'} and not $force){
+    return $data->{'authtrees'};
+  } elsif ($data->{'authtrees'}){
+    $hierarchies=$data->{'authtrees'};
+  } else {
+    my $record = AUTHgetauthority($dbh,$authid);
+    my $found;
+    my ($result,$total)=authoritysearch($dbh,['5503'],['and'],'',['='],[$authid],0,100);
+    if ($total){
+      foreach my $parentauthid (map { $_->{'authid'} } @$result){
+        my $parentrecord = AUTHgetauthority($dbh,$parentauthid);
+        #checking results
+        foreach my $field ($parentrecord->field('550')){
+          if (($field->subfield('3')) && ($field->subfield('3') eq $authid) && ($field->subfield('5')) && ($field->subfield('5') eq 'h')) {
+            my $localresult=$hierarchies;
+            my $trees;
+            $trees = BuildUnimarcHierarchies($parentauthid);
+            my @trees;
+            if ($trees=~/;/){
+              @trees = split(/;/,$trees);
+            } else {
+              push @trees, $trees;
+            }
+            foreach (@trees){
+              $_.= ",$authid";
+            }
+            @globalresult = (@globalresult,@trees);
+          }
+          $found=1;
+        }
+      }
+      $hierarchies=join(";",@globalresult);
+    }
+    #Unless there is no ancestor, I am alone.
+    $hierarchies="$authid" unless ($hierarchies);
+  }
+  AUTHsavetrees($authid,$hierarchies);
+  return $hierarchies;
+}
+
+sub BuildUnimarcHierarchy{
+	my $record = shift @_;
+    my $class = shift @_;
+    my $authid_constructed = shift @_;
+	my $authid=$record->subfield('250','3');
+    my %cell;
+	my $parents=""; my $children="";
+    my (@loopparents,@loopchildren);
+	foreach my $field ($record->field('550')){
+		if ($field->subfield('5') && $field->subfield('a')){
+		  if ($field->subfield('5') eq 'h'){
+            push @loopchildren, { "childauthid"=>$field->subfield('3'),"childvalue"=>$field->subfield('a')};
+		  }elsif ($field->subfield('5') eq 'g'){
+            push @loopparents, { "parentauthid"=>$field->subfield('3'),"parentvalue"=>$field->subfield('a')};
+		  }
+		# brothers could get in there with an else
+		}
+	}
+    $cell{"ifparents"}=1 if (scalar(@loopparents)>0);
+    $cell{"ifchildren"}=1 if (scalar(@loopchildren)>0);
+    $cell{"loopparents"}=\@loopparents if (scalar(@loopparents)>0);
+    $cell{"loopchildren"}=\@loopchildren if (scalar(@loopchildren)>0);
+    $cell{"class"}=$class;
+    $cell{"loopauthid"}=$authid;
+    $cell{"current_value"} =1 if $authid eq $authid_constructed;
+    $cell{"value"}=$record->subfield('250',"a");
+	return \%cell;
+}
+
+sub AUTHgetheader{
+	my $authid = shift @_;
+	my $sql= "SELECT * from auth_header WHERE authid = ?";
+	my $dbh=C4::Context->dbh;
+	my $rq= $dbh->prepare($sql);
+    $rq->execute($authid);
+	my $data= $rq->fetchrow_hashref;
+	return $data;
+}
+
+sub AUTHsavetrees{
+	my $authid = shift @_;
+	my $trees = shift @_;
+	my $sql= "UPDATE IGNORE auth_header set authtrees=? WHERE authid = ?";
+	my $dbh=C4::Context->dbh;
+	my $rq= $dbh->prepare($sql);
+    $rq->execute($trees,$authid);
+}
 
 END { }       # module clean-up code here (global destructor)
 
@@ -936,8 +1333,72 @@ Paul POULAIN paul.poulain@free.fr
 
 =cut
 
-# $Id: AuthoritiesMarc.pm,v 1.9.2.9 2005/12/01 17:30:26 tipaul Exp $
+# $Id: AuthoritiesMarc.pm,v 1.9.2.24 2006/08/10 12:42:33 tipaul Exp $
 # $Log: AuthoritiesMarc.pm,v $
+# Revision 1.9.2.24  2006/08/10 12:42:33  tipaul
+# warn commenting + some bugfixes I forgot to commit (for authorities editing)
+#
+# Revision 1.9.2.23  2006/08/03 00:50:34  kados
+# fix to bug 1130: Summary display broken for MARC21 Authorities
+#
+# Revision 1.9.2.22  2006/08/02 23:43:51  kados
+# partial fix to MARC21 authorities headings display
+#
+# Revision 1.9.2.21  2006/08/02 10:17:09  tipaul
+# some improvements in buildHerarchies (unimarc hierarchies)
+#
+# Revision 1.9.2.20  2006/07/31 13:29:14  tipaul
+# - adding a 3rd option to authority search (search on $a of on all subfields of main entry)
+# - removing automatic % add at the end of a search & adding a warning for the user
+#
+# Revision 1.9.2.19  2006/07/31 10:15:42  hdl
+# BugFixing : MARCdetail : displayin field values with ESCAPE=HTML  (in order to manage  '<''>' characters)
+#
+# Adding  Hierarchy display for authorities.
+# Please Note That it relies on the fact that authorities id are stored in $3 of authorities notice.
+# And Broader terms is supposed to be indicated by a g for 550$5 subfield, narrower term : an h for the same subfield.
+#
+# It CAN SURELY be generalised but only with a bunch of sytem preferences.
+#
+# I added the ability to do a search on ANY authtypecode.
+#
+# Revision 1.9.2.18  2006/07/25 12:30:51  tipaul
+# adding some informations to the array that is passed as result to an authority search : mainly, the tag_to_report & the $3 information (unimarc specific)
+#
+# Revision 1.9.2.17  2006/04/10 20:06:15  kados
+# Adding support for bulkauthimport of records where authid already exists.
+# This commit should be tested with other uses of AUTHaddauthority to ensure
+# it works.
+#
+# Revision 1.9.2.16  2006/04/03 12:52:50  tipaul
+# oups, sorry kados, I had removed something you wrote for MARC21 authorities...
+#
+# Revision 1.9.2.15  2006/03/30 14:20:03  tipaul
+# don't use + on a numeric value when you want to do a concat !!!
+#
+# Revision 1.9.2.14  2006/03/15 15:10:29  tipaul
+# added a new feature in summary building (for an authority)
+# If you enter [XXX*] ([250*] for example), the whole field will be displayed as it's saved. This will solve the problem with reordered subfields.
+#
+# Revision 1.9.2.13  2006/03/15 10:46:31  tipaul
+# removing hardcoded link in summary of authority (on $heading) : it can be set in the template (in the # of biblios column) :
+# <a href="/cgi-bin/koha/opac-search.pl?type=opac&amp;op=do_search&amp;marclist=<!-- TMPL_VAR NAME="biblio_fields" -->&amp;operator==&amp;value=<!-- TMPL_VAR NAME="authid" -->&amp;and_or=and&amp;excluding="><!-- TMPL_VAR NAME="used" --></a>  <!-- TMPL_VAR NAME="used" -->
+#
+# that's what I did for css templates, it work like a charm. It's better I think because when the library defines it's own summary, the hardcoded link didn't appear.
+#
+# Revision 1.9.2.12  2006/03/09 01:45:14  kados
+# Refining list of appropriate subfields to display for the authorized
+# heading.
+#
+# Revision 1.9.2.11  2006/03/08 15:17:09  tipaul
+# fixing some UNIMARC behaviour + removing some hardcoded strings
+#
+# Revision 1.9.2.10  2006/03/06 19:11:55  kados
+# Fixes buggy use of ISBD for summary in Authorities display. Previously,
+# it was not possible to properly display repeated tags/subfields in the
+# correct order. This code uses the MARC21 guidelines for display of the
+# main heading, see and see also listings.
+#
 # Revision 1.9.2.9  2005/12/01 17:30:26  tipaul
 # no need to do a search on an authority when the authority has no MARC field (like EDITORS pseudo authority)
 #
