@@ -30,7 +30,7 @@ use C4::Biblio;
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 # set the version for version checking
-$VERSION = do { my @v = '$Revision: 1.99.2.11 $' =~ /\d+/g;
+$VERSION = do { my @v = '$Revision: 1.99.2.15 $' =~ /\d+/g;
           shift(@v) . "." . join("_", map {sprintf "%03d", $_ } @v); };
 
 =head1 NAME
@@ -1306,12 +1306,10 @@ sub ItemInfo {
 					WHERE items.biblionumber = ?
 					AND biblioitems.biblioitemnumber = items.biblioitemnumber
 					AND biblio.biblionumber = items.biblionumber";
-# buggy : opac & librarian interface can show the same info level & itemstatus should not be hardcoded
-# 	if ($type ne 'intra'){
-# 		$query .= " and ((items.itemlost<>1 and items.itemlost <> 2)
-# 		or items.itemlost is NULL)
-# 		and (wthdrawn <> 1 or wthdrawn is NULL)";
-# 	}
+	if ($type ne 'intra' && C4::Context->preference('hidelostitems')) {
+		$query .= " and (items.itemlost<1 or items.itemlost is NULL)
+		and (wthdrawn <> 1 or wthdrawn is NULL)";
+	}
 	$query .= " order by items.homebranch, items.dateaccessioned desc";
 	my $sth=$dbh->prepare($query);
 	$sth->execute($biblionumber);
@@ -1340,9 +1338,10 @@ sub ItemInfo {
 		if ($datedue eq ''){
 	#	$datedue="Available";
 			my ($restype,$reserves)=C4::Reserves2::CheckReserves($data->{'itemnumber'});
-			if ($restype) {
-				$datedue=$restype;
-			}
+            $data->{reservestatus} = $restype;
+# 			if ($restype) {
+# 				$datedue=$restype;
+# 			}
 		}
 		$isth->finish;
 	#get branch information.....
@@ -1541,7 +1540,7 @@ sub bibdata {
 	chop $data->{'additionalauthors'};
 	chop $data->{'additionalauthors'};
 	# handle ISBN : reintroduce - if there are none
-	$data->{'isbn'} = DisplayISBN($data->{'isbn'});
+	$data->{'isbn'} = DisplayISBN($data->{'isbn'}) if $data->{isbn};
 	$sth->finish;
 	return($data);
 } # sub bibdata
@@ -1704,7 +1703,7 @@ sub itemissues {
     # gotten, the user might get a blank page. It would be much better
     # to at least print an error message. The easiest way to do this
     # is to set $SIG{__DIE__}.
-    my $sth   = $dbh->prepare("Select * from items where
+    my $sth   = $dbh->prepare("Select *,items.notforloan as itemnotforloan from items where
 items.biblioitemnumber = ?")
       || die $dbh->errstr;
     my $i     = 0;
@@ -1721,7 +1720,7 @@ items.biblioitemnumber = ?")
         # fetchrow_hashref() can fail for any number of reasons (e.g.,
         # database server crash), not just because no items match the
         # search criteria.
-        my $sth2   = $dbh->prepare("select * from issues,borrowers
+        my $sth2   = $dbh->prepare("select *, items.notforloan as itemnotforloan from items,issues,borrowers
 where itemnumber = ?
 and returndate is NULL
 and issues.borrowernumber = borrowers.borrowernumber");
@@ -1757,6 +1756,17 @@ and issues.borrowernumber = borrowers.borrowernumber");
         } # for
 
         $sth2->finish;
+	# get notforloan complete status if applicable
+	my $sthnflstatus = $dbh->prepare('select authorised_value from marc_subfield_structure where kohafield="items.notforloan"');
+	$sthnflstatus->execute;
+	my ($authorised_valuecode) = $sthnflstatus->fetchrow;
+	if ($authorised_valuecode) {
+		$sthnflstatus = $dbh->prepare("select lib from authorised_values where category=? and authorised_value=?");
+		$sthnflstatus->execute($authorised_valuecode,$data->{itemnotforloan});
+		my ($lib) = $sthnflstatus->fetchrow;
+		$data->{notforloantext} = $lib;
+		warn "NOT FOR LOAN".$lib.$data->{itemnotforloan};
+	}
         $results[$i] = $data;
         $i++;
     }
