@@ -176,10 +176,6 @@ sub catalogsearch {
 	#		from marc_subfield_table as m1, marc_subfield_table as m2
 	#		where m1.bibid=m2.bibid and
 	#		(m1.subfieldvalue like "Des%" and m2.subfieldvalue like "27%")
-
-	# last minute stripping out of stuff
-	# doesn't work @$value =~ s/\'/ /;
-	# @$value = map { $_ =~ s/\'/ /g } @$value;
 	
 	# "Normal" statements
 	my @normal_tags = ();
@@ -194,16 +190,10 @@ sub catalogsearch {
 	my $any_not = 0;
 	$orderby = "biblio.title" unless $orderby;
 	$desc_or_asc = "ASC" unless $desc_or_asc;
-	#last minute stripping out of ' and ,
-# paul : quoting, it's done a few lines lated.
-# 	foreach $_ (@$value) {
-# 		$_=~ s/\'/ /g;
-# 		$_=~ s/\,/ /g;
-# 	}
 
-# the item.notforloan contains an integer. Every value <>0 means "book unavailable for loan".
-# but each library can have it's own table of meaning for each value. Get them
-# 1st search if there is a list of authorised values connected to items.notforloan
+	# the item.notforloan contains an integer. Every value <>0 means "book unavailable for loan".
+	# but each library can have it's own table of meaning for each value. Get them
+	# 1st search if there is a list of authorised values connected to items.notforloan
 	my $sth = $dbh->prepare('select authorised_value from marc_subfield_structure where kohafield="items.notforloan"');
 	$sth->execute;
 	my %notforloanstatus;
@@ -215,13 +205,12 @@ sub catalogsearch {
 			$notforloanstatus{$authorised_value} = $lib?$lib:$authorised_value;
 		}
 	}
-	for(my $i = 0 ; $i <= $#{$value} ; $i++)
-	{
+	for(my $i = 0 ; $i <= $#{$value} ; $i++) {
 		# replace * by %
 		@$value[$i] =~ s/\*/%/g;
 		# remove % at the beginning
 		@$value[$i] =~ s/^%//g;
-	    @$value[$i] =~ s/(\.|\?|\:|\!|\'|,|\-|\"|\(|\)|\[|\]|\{|\}|\/|--)/ /g if @$operator[$i] eq "contains";
+		@$value[$i] =~ s/(\.|\?|\:|\!|\'|,|\-|\"|\(|\)|\[|\]|\{|\}|\/|--)/ /g if @$operator[$i] eq "contains";
 		if(@$excluding[$i])	# NOT statements
 		{
 			$any_not = 1;
@@ -335,7 +324,58 @@ sub catalogsearch {
 			push @result,$bibid;
 		}
 	}
+if (C4::Context->preference("sortbynonfiling")) {
+#~~#~~# next part added by SH Modified by JF
+# This code resorts the biblio.title results eliminating the
+# marc non-filing chars
+#this is a big problem perf wise
+#
+# A better way is to create a new column in marc_word called
+# subfield_string.  it's made without non-filing chars and you
+# sort by it ...
+# or maybe you just quer marc_subfield_table in the origiinal query.
+#
+        if ($orderby eq "biblio.title") {    #don't sort by title if another orderby is specified
+        		my ($tag,$subfieldcode) = MARCfind_marc_from_kohafield($dbh,'biblio.title','');
+                my $tsth = $dbh->prepare("SELECT title, tag_indicator FROM marc_biblio, marc_subfield_table, biblio
+                                                         WHERE marc_biblio.bibid = ?
+                                                         AND marc_subfield_table.bibid=marc_biblio.bibid
+                                                         AND biblio.biblionumber=marc_biblio.biblionumber
+                                                         AND tag=? and subfieldcode=?");
+                my %titles = ();    #to hold title data for sorting
+                my $sorttitle;
+                my $bibidno;
+                my @resultnew = ();    #clear previous list of bibids
 
+                foreach my $bibidno (@result) {
+                        $tsth->execute($bibidno,$tag,$subfieldcode);
+                        my $titledat = $tsth->fetchrow_hashref;
+                        $sorttitle = lc($titledat->{title});
+#                         warn "Here's the sorttitle beforehand: $titledat->{tag_indicator}=".$sorttitle;
+                        if (length($titledat->{tag_indicator})==2) {
+                                my $filechar=substr($titledat->{tag_indicator},1,1);
+                                $sorttitle = substr($sorttitle,$filechar,-1,"");  #remove nonfiling characters
+#                                warn "Here it is afterwards:".$sorttitle;
+                        }
+                        $titles{$bibidno} = $sorttitle; #} = $bibidno;
+#                       push @resultnew, $titles{$sorttitle};
+# sorttitle is key ... biblionumber is value
+# but it needs to be the other way around
+                }
+
+                $tsth->finish;
+                #while ( my ($k,$v) = each %titles ) { warn "$k => $v\n"; }
+
+
+                foreach my $bibses (sort {$titles{$a} cmp $titles{$b}} (keys(%titles))) {
+                        push @resultnew, $bibses;
+                }
+                #@resultnew = sort { $titles{$a} <=> $titles{$b} } keys %titles;
+                @result = ();
+                @result = @resultnew;
+        }
+#~~#~~#  above added by SH
+}    
 	# we have bibid list. Now, loads title and author from [offset] to [offset]+[length]
 	my $counter = $offset;
 	# HINT : biblionumber as bn is important. The hash is fills biblionumber with items.biblionumber.
@@ -352,12 +392,33 @@ sub catalogsearch {
 	my $oldline;
 	my ($oldbibid, $oldauthor, $oldtitle);
 	my $sth_itemCN;
-	if (C4::Context->preference('hidelostitem')) {
-		$sth_itemCN = $dbh->prepare("select items.* from items where biblionumber=? and (itemlost = 0 or itemlost is NULL) order by homebranch");
+	my $userenv=C4::Context->userenv;
+# 	warn "FLAGS : ".($userenv->{flags} & 2**9);
+    # hide lost items if :
+    # - the preference hidelostitems is set
+    # - the user not logged in, or is not a superlibrarian or a librarian with cataloguing permissions
+    # otherwise, show everything
+	if (!C4::Context->preference('hidelostitems') or ($userenv && ($userenv->{'flags'} eq 1 or ($userenv->{'flags'} & 2**9)) )) {
+	# show everything
+		$sth_itemCN = $dbh->prepare("
+  SELECT items.holdingbranch, items.location, items.itemcallnumber, count(*) AS cnt, items.itemnumber, items.notforloan
+  FROM items 
+  WHERE biblionumber=? 
+  GROUP BY items.holdingbranch, items.location, items.itemcallnumber 
+  ORDER BY homebranch");
 	} else {
-		$sth_itemCN = $dbh->prepare("select items.* from items where biblionumber=? order by homebranch");
+	# hide lost items
+		$sth_itemCN = $dbh->prepare("
+  SELECT items.holdingbranch, items.location, items.itemcallnumber, count(*) AS cnt, items.itemnumber, items.notforloan
+  FROM items 
+  WHERE biblionumber=? AND (itemlost = 0 OR itemlost IS NULL)
+  GROUP BY items.holdingbranch, items.location, items.itemcallnumber
+  ORDER BY homebranch");
 	}
-	my $sth_issue = $dbh->prepare("select date_due,returndate from issues where itemnumber=?");
+	my $sth_issue = $dbh->prepare("
+  SELECT date_due,returndate 
+  FROM issues 
+  WHERE itemnumber=?");
 	# parse all biblios between start & end.
 	while (($counter <= $#result) && ($counter <= ($offset + $length))) {
 		# search & parse all items & note itemcallnumber
@@ -385,24 +446,27 @@ sub catalogsearch {
 			# then all other fields in the main array
 			
 			# search if item is on loan
-			my $date_due;
-			$sth_issue->execute($item->{itemnumber});
-			while (my $loan = $sth_issue->fetchrow_hashref) {
-				if ($loan->{date_due} and !$loan->{returndate}) {
-					$date_due = $loan->{date_due};
-				}
-			}
 			# store this item
 			my %lineCN;
 			$lineCN{holdingbranch} = $item->{holdingbranch};
 			$lineCN{itemcallnumber} = $item->{itemcallnumber};
 			$lineCN{location} = $item->{location};
-			$lineCN{date_due} = format_date($date_due);
-			$lineCN{notforloan} = $notforloanstatus{$line->{notforloan}} if ($line->{notforloan}); # setting not forloan if itemtype is not for loan
-			$lineCN{notforloan} = $notforloanstatus{$item->{notforloan}} if ($item->{notforloan}); # setting not forloan it this item is not for loan
-			$notforloan=0 unless ($item->{notforloan} or $item->{wthdrawn} or $item->{itemlost});
+			$lineCN{cnt} = $item->{cnt} unless ($item->{cnt}==1);
+            if ($item->{cnt}==1){
+              my $date_due;
+              $sth_issue->execute($item->{itemnumber});
+              while (my $loan = $sth_issue->fetchrow_hashref) {
+                  if ($loan->{date_due} and !$loan->{returndate}) {
+                      $date_due = $loan->{date_due};
+                  }
+              }
+              $lineCN{date_due} = format_date($date_due) ;
+              $lineCN{notforloan} = $notforloanstatus{$item->{notforloan}} if ($item->{notforloan}); # setting not forloan it this item is not for loan
+            }
+            $notforloan=0 unless ($item->{notforloan} or $item->{wthdrawn} or $item->{itemlost});
+			$lineCN{notforloan} = $notforloanstatus{$line->{notforloan}} if ($line->{notforloan} and not $lineCN{notforloan}); # setting not forloan if itemtype is not for loan
 			push @CNresults,\%lineCN;
-			$totalitems++;
+			$totalitems+=$item->{cnt};
 		}
 		# save the biblio in the final array, with item and item issue status
 		my %newline;
@@ -435,6 +499,7 @@ sub catalogsearch {
 	return (\@finalresult, $nbresults);
 }
 
+
 # Creates the SQL Request
 
 sub create_request {
@@ -457,6 +522,7 @@ sub create_request {
 					$sql_tables .= "marc_subfield_table as m$nb_table,";
 					$sql_where1 .= "(m1.subfieldvalue like ".$dbh->quote("@$value[$i]%");
 					if (@$tags[$i]) {
+                        @$tags[$i] = $dbh->quote(@$tags[$i]) unless @$tags[$i]=~ /^'/;
 						$sql_where1 .=" and concat(m1.tag,m1.subfieldcode) in (@$tags[$i])";
 					}
 					$sql_where1.=")";
@@ -464,6 +530,7 @@ sub create_request {
 					$sql_tables .= "marc_word as m$nb_table,";
 					$sql_where1 .= "(m1.word  like ".$dbh->quote("@$value[$i]");
 					if (@$tags[$i]) {
+                        @$tags[$i] = $dbh->quote(@$tags[$i]) unless @$tags[$i]=~ /^'/;
 						 $sql_where1 .=" and m1.tagsubfield in (@$tags[$i])";
 					}
 					$sql_where1.=")";
@@ -471,6 +538,7 @@ sub create_request {
 					$sql_tables .= "marc_subfield_table as m$nb_table,";
 					$sql_where1 .= "(m1.subfieldvalue @$operator[$i] ".$dbh->quote("@$value[$i]");
 					if (@$tags[$i]) {
+                        @$tags[$i] = $dbh->quote(@$tags[$i]) unless @$tags[$i]=~ /^'/;
 						 $sql_where1 .=" and concat(m1.tag,m1.subfieldcode) in (@$tags[$i])";
 					}
 					$sql_where1.=")";
@@ -481,6 +549,7 @@ sub create_request {
 					$sql_tables .= "marc_subfield_table as m$nb_table,";
 					$sql_where1 .= "@$and_or[$i] (m$nb_table.subfieldvalue like ".$dbh->quote("@$value[$i]%");
 					if (@$tags[$i]) {
+                        @$tags[$i] = $dbh->quote(@$tags[$i]) unless @$tags[$i]=~ /^'/;
 					 	$sql_where1 .=" and concat(m$nb_table.tag,m$nb_table.subfieldcode) in (@$tags[$i])";
 					}
 					$sql_where1.=")";
@@ -491,6 +560,7 @@ sub create_request {
 						$sql_tables .= "marc_word as m$nb_table,";
 						$sql_where1 .= "@$and_or[$i] (m$nb_table.word like ".$dbh->quote("@$value[$i]");
 						if (@$tags[$i]) {
+                            @$tags[$i] = $dbh->quote(@$tags[$i]) unless @$tags[$i]=~ /^'/;
 							$sql_where1 .=" and m$nb_table.tagsubfield in(@$tags[$i])";
 						}
 						$sql_where1.=")";
@@ -498,6 +568,7 @@ sub create_request {
 					} else {
 						$sql_where1 .= "@$and_or[$i] (m$nb_table.word like ".$dbh->quote("@$value[$i]");
 						if (@$tags[$i]) {
+                            @$tags[$i] = $dbh->quote(@$tags[$i]) unless @$tags[$i]=~ /^'/;
 							$sql_where1 .="  and m$nb_table.tagsubfield in (@$tags[$i])";
 						}
 						$sql_where1.=")";
@@ -508,6 +579,7 @@ sub create_request {
 					$sql_tables .= "marc_subfield_table as m$nb_table,";
 					$sql_where1 .= "@$and_or[$i] (m$nb_table.subfieldvalue @$operator[$i] ".$dbh->quote(@$value[$i]);
 					if (@$tags[$i]) {
+                        @$tags[$i] = $dbh->quote(@$tags[$i]) unless @$tags[$i]=~ /^'/;
 					 	$sql_where1 .="  and concat(m$nb_table.tag,m$nb_table.subfieldcode) in (@$tags[$i])";
 					}
 					$sql_where2 .= "m1.bibid=m$nb_table.bibid and ";
@@ -583,13 +655,28 @@ sub getMARCsubjects {
     my ($dbh, $bibid, $marcflavour) = @_;
 	my ($mintag, $maxtag);
 	if ($marcflavour eq "MARC21") {
-	        $mintag = "600";
+        $mintag = "600";
 		$maxtag = "699";
 	} else {           # assume unimarc if not marc21
 		$mintag = "600";
 		$maxtag = "699";
 	}
-	my $sth=$dbh->prepare("SELECT subfieldvalue,subfieldcode,tagorder,tag FROM marc_subfield_table WHERE bibid=? AND tag BETWEEN ? AND ? ORDER BY tagorder,subfieldorder");
+	my $sth=$dbh->prepare("SELECT `subfieldvalue`,`subfieldcode`,`tagorder`,`tag` FROM `marc_subfield_table` WHERE `bibid`= ? AND `subfieldcode` NOT IN ('2','4','6','8') AND `tag` BETWEEN ? AND ? ORDER BY `tagorder`,`subfieldorder`");
+	# Subfield exclusion for $2, $4, $6, $8 protects against searching for
+	# variant data in otherwise invariant authorised subject headings when all
+	# returned subfields are used to form a query for matching subjects.  One
+	# example is the use of $2 in MARC 21 where the value of $2 changes for
+	# different editions of the thesaurus used, even where the subject heading
+	# is otherwise the same.  There is certainly a better fix for many cases
+	# where the value of the subfield may be parsed for the invariant data.  
+	# More complete display values may also be separated from query values
+	# containing only the actual invariant authorised subject headings.  More
+	# coding is required for careful value parsing, or display and query
+	# separation; instead of blanket subfield exclusion.
+	# 
+	# As implemented, $3 is passed and might still pose a problem.  Passing $3
+	# could have benefits for some proper use of $3 for UNIMARC, however, might
+	# restrict query usage to a given material type.  -- thd
 
 	$sth->execute($bibid,$mintag,$maxtag);
 
@@ -601,11 +688,12 @@ sub getMARCsubjects {
 	my $lasttag;
 	my ($subfieldvalue,$subfieldcode,$tagorder,$tag);
 	while (($subfieldvalue,$subfieldcode,$tagorder,$tag)=$sth->fetchrow) {
-		$lasttag=$tag if $tag;
+	#warn "IN MARCSUBJECTS $subfieldvalue $subfieldcode $tagorder $tag\n";
 		if ($activetagorder && $tagorder != $activetagorder) {
+	#	warn "ACTIVETAGORDER".$activetagorder;
 			$subject=~ s/ -- $//;
 			$marcsubjct = {MARCSUBJCT => $subject,
-							link => $tag."9",
+							link => $lasttag."9",
 							linkvalue => $field9,
 							};
 			push @marcsubjcts, $marcsubjct;
@@ -615,21 +703,26 @@ sub getMARCsubjects {
 		}
 		if ($subfieldcode eq 9) {
 			$field9=$subfieldvalue;
+		} elsif ($subfieldcode eq (3 || 5)) {
+			$subject .= $subfieldvalue . " ";
 		} else {
-			$subject .= $subfieldvalue." -- ";
+			$subject .= $subfieldvalue . " -- ";
 		}
 		$activetagorder=$tagorder;
+		$lasttag=$tag if $tag;
 	}
-	$marcsubjct = {MARCSUBJCT => $subject,
+	if ($subject) {
+		$subject=~ s/ -- $//;
+		$marcsubjct = {MARCSUBJCT => $subject,
 					link => $lasttag."9",
 					linkvalue => $field9,
 					};
-	push @marcsubjcts, $marcsubjct;
-
+		push @marcsubjcts, $marcsubjct;
+	}
 	$sth->finish;
 
 	my $marcsubjctsarray=\@marcsubjcts;
-        return $marcsubjctsarray;
+	return $marcsubjctsarray;
 }  #end getMARCsubjects
 
 END { }       # module clean-up code here (global destructor)
